from osgeo import gdal, gdal_array
from pcraster import *
import numpy as np
import os
from datetime import datetime
import sys

###########################################################################################################################
#
# GLOFAS seasonal forecast processing script 2: 
# 
# This script, for one forecast (weekly-averaged river flow) for each week out to 17 weeks (4 months), calculates
# the probability of exceeding the 80th, or falling below the 20th, percentile of the reference climatology for the same week
# (e.g. probability of the forecast for 1-7 March exceeding the 80th percentile of climatology for the 1-7 March
#
# This is saved as a pcraster map for later use
#
# (file paths have been changed)
###########################################################################################################################

def compute_exceedance_probs(year, month):
    
    en = 51 #number of ensemble members
		  
    #where are we saving the output probability files?:
    savedir = "/weekly_exc_probs/savedir/"+str(year)+str(month).zfill(2)+"/"
    if not os.path.exists(savedir):
        os.makedirs(savedir)
      
    #directory containing the percentile values calculated from the reference climatology
    pctldir = "/archive/ref_clim/percentiles/"+str(month).zfill(2)+"/"
    
    #directory containing the forecast files (weekly-averaged gridded discharge forecast)
    fcstdir = "/weekly_averages/save_dir/"+str(year)+str(month).zfill(2)+"/" 
        
        
    for week in range(0,17):
        
        w2 = int(week+1)
        
        #read the pcraster format percentile data files into arrays so we can process them:
        
        pctl80file = pctldir+"clim_m"+str(month).zfill(2)+"_w"+str(w2).zfill(2)+"_p80.map"
        ffpctl80 = gdal.Open(pctl80file)
        rowspctl80 = ffpctl80.RasterYSize
        colspctl80 = ffpctl80.RasterXSize
        bandpctl80 = ffpctl80.GetRasterBand(1)
        pctl80data = bandpctl80.ReadAsArray(0,0,colspctl80,rowspctl80)
        
        pctl20file = pctldir+"clim_m"+str(month).zfill(2)+"_w"+str(w2).zfill(2)+"_p20.map"
        ffpctl20 = gdal.Open(pctl20file)
        rowspctl20 = ffpctl20.RasterYSize
        colspctl20 = ffpctl20.RasterXSize
        bandpctl20 = ffpctl20.GetRasterBand(1)
        pctl20data = bandpctl20.ReadAsArray(0,0,colspctl20,rowspctl20)
      
        #empty arrays to store exceedance probabilities, for all grid cells, for this week of the forecast only    
        numcols = 3600
        numrows = 1500
    
        probexc80 = np.zeros((numrows,numcols)) 
        probbel20 = np.zeros((numrows,numcols)) 
    
        #for each ensemble member
        for e in range(en):
        
            #read the pcraster format forecast data (weekly averaged gridded discharge) file into an array so we can process it
            fcstfile = fcstdir+"e"+str(e).zfill(2)+'/wa_gridded_dis_fcst_e'+str(e).zfill(2)+'_wk'+str(week).zfill(2)+'.map'
            fffcst = gdal.Open(fcstfile)
            rowsfcst = fffcst.RasterYSize
            colsfcst = fffcst.RasterXSize
            bandfcst = fffcst.GetRasterBand(1)
            fcstdata = bandfcst.ReadAsArray(0,0,colsfcst,rowsfcst)

            #for each grid cell
            for r in range(numrows):
                    for c in range(numcols):
                            #if the forecast river flow exceeds the 80th percentile climatology, add 1 to the count for probability of exceeding 80th percentile
                            if fcstdata[r,c] > pctl80data[r,c]: 
                                    probexc80[r,c] += 1
                            #if the forecast river flow is below the 20th percentile of climatology, add 1 to the count for probability of exceeding 20th percentile
                            elif fcstdata[r,c] < pctl20data[r,c]: 
                                    probbel20[r,c] += 1
                                
        #now we have the count of the number of ensemble members exceeding the threshold, so turn it into the % of ensemble members (probability of exceedance for that week)                            
        probexc80 = (probexc80/en)*100
        probbel20 = (probbel20/en)*100
    
	
        #save as a pcraster map
        driver = gdal.GetDriverByName('GTiff')
    
        print 'saving tiff files'
        dst_filename80 = savedir+'wa_gridded_prob_exc_P80_'+str(month).zfill(2)+'_wk'+str(week).zfill(2)+'.tiff'
        dataset80 = driver.Create(dst_filename80,numcols, numrows, 1,gdal.GDT_Float32)
        dataset80.GetRasterBand(1).WriteArray(probexc80)
    
        dst_filename20 = savedir+'wa_gridded_prob_exc_P20_'+str(month).zfill(2)+'_wk'+str(week).zfill(2)+'.tiff'
        dataset20 = driver.Create(dst_filename20,numcols, numrows, 1,gdal.GDT_Float32)
        dataset20.GetRasterBand(1).WriteArray(probbel20)

        geotrans=fffcst.GetGeoTransform() 
        proj=fffcst.GetProjection()          
        
        dataset80.SetGeoTransform(geotrans)
        dataset80.SetProjection(proj)
        dataset80.FlushCache()
        dataset80=None

        dataset20.SetGeoTransform(geotrans)
        dataset20.SetProjection(proj)
        dataset20.FlushCache()
        dataset20=None

if __name__ == '__main__':
  compute_exceedance_probs(int(sys.argv[1]), int(sys.argv[2]))
    

    
    
    
    
    
    
    
    
    
